
import sys
sys.path.insert(0,'/home/zhangtian/dev/algo-client-api/src/')
import time
from aats.trade_service import TradeService
from aats.base_strategy import BaseStrategy
from aats.base.order import Order
from aats.base.const import OrderType, Side
from aats.market_service import NetWorkType

sym_cid_map = {
    1001: "BTCUSDT.BINANCE",
    1002: "ETHUSDT.BINANCE",
    1003: "DOGEUSDT.BINANCE"
    }

## define your strategy
class SimpleDataPrinterStrategy(BaseStrategy):
    """
    Naive Market Data Printer Strategy: print out quote and tick data on the given time frequency
    """
    def __init__(self, msecs):
        self.msecs = msecs
        self.start_time = time.time_ns()
        self.next_fire_time = self.start_time + self.msecs * 1e6

    def onQuote(self, msg):
        current_time = time.time_ns()
        if current_time > self.next_fire_time:
            self.next_fire_time = current_time + self.msecs * 1e6
            print(msg)

    def onTick(self, msg):
        current_time = time.time_ns()
        if current_time > self.next_fire_time:
            self.next_fire_time = current_time + self.msecs * 1e6
            print(msg)


# init trade engine
trade_service = TradeService(sym_cid_map)

# setup public config
trade_service.set_md_connection(ip='127.0.0.1', port=15004, send_type = NetWorkType.P2P)  # for P2P mode, port is required to use the port from market server
trade_service.add_md_symbol(symbol='BTCUSDT', exchange='BINANCE', level_num=5)
# trade_service.add_md_symbol(symbol='DOGEUSDT', exchange='BINANCE')

# setup private config
trade_service.set_ts_connection(ip='127.0.0.1', port=8060)  # default setting
trade_service.config_exchange(exchange='BINANCE', trade_type='sandbox')
trade_service.set_apikey(exchange="BINANCE", 
                        key="02SvhZEYG1p92JWdekP75XQKayqfLxmjHWNEfWU1KrCPjJ5xrLcOU1YHZ5SUBVFA", 
                        secret="iKnZvDKMGQuEINhjDX8gbIVJDLl48fV6GFLL5gcFT8Sfj9yxGrnP7uFm7AAVWeFP", 
                        password="",
                        subaccount="")
trade_service.set_fee('BINANCE', 0.0003, 0)
trade_service.add_trade_symbol(symbol='BTCUSDT', exchange='BINANCE', level_num=5)

# setup your strategy
# my_strategy = SimpleMakerStrategy()
my_strategy = SimpleDataPrinterStrategy(msecs=5000)
trade_service.add_strategy(my_strategy)

# start run
try:
    trade_service.run()
except BaseException as e:
    if isinstance(e, KeyboardInterrupt):
        # trade_service.stop()
        print("close strategy and close open orders")
        # trade_service.manager.stop()
        my_strategy.close()
        time.sleep(1)
        print("trade closed")    


from aats.base_strategy import BaseStrategy
from aats.base.order import Order
from aats.base.const import OrderType, OrderStatus, Side
import time

class SimpleTakerStrategy(BaseStrategy):
    """
    Naive Taker Strategy: place IOC buy order on fixed time interval
    """
    def __init__(self, msecs):
        self.msecs = msecs
        self.start_time = time.time_ns()
        self.next_fire_time = self.start_time + self.msecs * 1e6

    def onNotify(self, No_type):
        current_time = time.time_ns()
        if current_time > self.next_fire_time:
            self.next_fire_time = current_time + self.msecs * 1e6
            self.onTimer()

    def onTimer(self):
        qpx = self.si[self.trade_cid].ask_px
        size = 0.001
        new_order = Order(exchange=self.trade_exchange, 
                            symbol=self.trade_symbol, 
                            price=qpx, 
                            size=size, 
                            side=Side.BUY.value, 
                            tif=OrderType.IOC.value)
        self.send_order(new_order)
        print(f"send order {new_order}")




from aats.base_strategy import BaseStrategy
from aats.base.order import Order
from aats.base.const import OrderType, OrderStatus, Side

class SimpleMakerStrategy(BaseStrategy):
    """
    Naive Maker Strategy: quote on best bid price and ask price
    """
    def __init__(self, *args):
        self.active_orders = {Side.BUY.value: [], Side.SELL.value: []}
        self.pending_orders = {Side.BUY.value: [], Side.SELL.value: []}
        self.cancelling_orders = []

    def close(self):
        for order in self.active_orders:
            self.send_cancel_order(order.order_id)
        
    def onNotify(self, no_type):
        if no_type == 1:
            bid_tgt_px = self.si[self.trade_cid].bid_px * (1-0.0001)
            ask_tgt_px = self.si[self.trade_cid].ask_px * (1+0.0001)
            self.maintain_order(Side.BUY.value, bid_tgt_px)
            self.maintain_order(Side.SELL.value, ask_tgt_px)

    
    def maintain_order(self, side, qpx):
        if len(self.active_orders[side]) >= 1:
            order = self.active_orders[side][0]
            opx = order.price
            if abs(qpx/opx-1) > 0.0001:
                if order.order_id not in self.cancelling_orders:
                    print(f"send cancel order id={order.order_id}")
                    self.cancelling_orders.append(order.order_id)
                    self.send_cancel_order(order.order_id)
        elif len(self.pending_orders[side]) == 0:
            size = 0.001
            order = Order(exchange=self.trade_exchange, 
                            symbol=self.trade_symbol, 
                            price=qpx, 
                            size=size, 
                            side=side, 
                            tif=OrderType.POS.value
                            )
            self.pending_orders[side].append(order)
            print(f"send order id {order.order_id}: px {qpx}, side {side}")
            self.send_order(order)

    
    def onOrderCreated(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.pending_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.ADD
                self.active_orders[side].append(order)
                self.pending_orders[side].remove(order)
                return

        
    def onOrderAcked(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.ACK
                return

    def onOrderRejected(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.RJD
                self.pending_orders[side].remove(order)
                return

    def onOrderCancelCreated(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CXL
                self.cancelling_orders.remove(order_id)
                return

    def onOrderCancelAcked(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CAK
                return

    def onOrderCancelRejected(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CRD
                self.active_orders[side].remove(order)
                return

    def onOrderExec(self, msg):
        order_id = msg.get("order").get("order_id")
        fill_size = float(msg.get("order").get("fill_size"))
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.fill_size += fill_size
                if abs(order.fill_size-order.size) < 1e-8:
                    order.order_status = OrderStatus.EXE
                    self.active_orders[side].remove(order)
                    return

    def onOrderCanceled(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CXD
                self.active_orders[side].remove(order)
                return

    def onOrderClosed(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CLD
                self.active_orders[side].remove(order)
                return



import sys
sys.path.insert(0,'/home/zhangtian/dev/algo-client-api/src/')
import time
from aats.trade_service import TradeService
from aats.base_strategy import BaseStrategy
from aats.base.order import Order
from aats.base.const import OrderType, OrderStatus, Side
from aats.market_service import NetWorkType


sym_cid_map = {
    1001: "BTCUSDT.BINANCE",
    1002: "ETHUSDT.BINANCE",
    1003: "DOGEUSDT.BINANCE"
    }

## define your strategy
class SimpleMakerStrategy(BaseStrategy):
    """
    Naive Maker Strategy: quote on best bid price and ask price
    """
    def __init__(self, *args):
        self.active_orders = {Side.BUY.value: [], Side.SELL.value: []}
        self.pending_orders = {Side.BUY.value: [], Side.SELL.value: []}
        self.cancelling_orders = []

    def close(self):
        for order in self.active_orders:
            self.send_cancel_order(order.order_id)
        
    def onNotify(self, no_type):
        if no_type == 1:
            bid_tgt_px = self.si[self.trade_cid].bid_px * (1-0.0001)
            ask_tgt_px = self.si[self.trade_cid].ask_px * (1+0.0001)
            self.maintain_order(Side.BUY.value, bid_tgt_px)
            self.maintain_order(Side.SELL.value, ask_tgt_px)

    
    def maintain_order(self, side, qpx):
        if len(self.active_orders[side]) >= 1:
            order = self.active_orders[side][0]
            opx = order.price
            if abs(qpx/opx-1) > 0.0001:
                if order.order_id not in self.cancelling_orders:
                    print(f"send cancel order id={order.order_id}")
                    self.cancelling_orders.append(order.order_id)
                    self.send_cancel_order(order.order_id)
        elif len(self.pending_orders[side]) == 0:
            size = 0.001
            order = Order(exchange=self.trade_exchange, 
                            symbol=self.trade_symbol, 
                            price=qpx, 
                            size=size, 
                            side=side, 
                            tif=OrderType.POS.value
                            )
            self.pending_orders[side].append(order)
            print(f"send order id {order.order_id}: px {qpx}, side {side}")
            self.send_order(order)

    
    def onOrderCreated(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.pending_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.ADD
                self.active_orders[side].append(order)
                self.pending_orders[side].remove(order)
                return

        
    def onOrderAcked(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.ACK
                return

    def onOrderRejected(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.RJD
                if order in self.pending_orders[side]:
                    self.pending_orders[side].remove(order)
                return

    def onOrderCancelCreated(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CXL
                self.cancelling_orders.remove(order_id)
                return

    def onOrderCancelAcked(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CAK
                return

    def onOrderCancelRejected(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CRD
                self.active_orders[side].remove(order)
                return

    def onOrderExec(self, msg):
        order_id = msg.get("order").get("order_id")
        fill_size = float(msg.get("order").get("fill_size"))
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.fill_size += fill_size
                if abs(order.fill_size-order.size) < 1e-8:
                    order.order_status = OrderStatus.EXE
                    self.active_orders[side].remove(order)
                    return

    def onOrderCanceled(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CXD
                self.active_orders[side].remove(order)
                return

    def onOrderClosed(self, msg):
        order_id = msg.get("order").get("order_id")
        side =  msg.get("order").get("side")
        for order in self.active_orders[side]:
            if order.order_id == order_id:
                order.order_status = OrderStatus.CLD
                self.active_orders[side].remove(order)
                return

# init trade engine
trade_service = TradeService(sym_cid_map)

# setup public config
trade_service.set_md_connection(ip='127.0.0.1', port=15004, send_type = NetWorkType.P2P)  # for P2P mode, port is required to use the port from market server
trade_service.add_md_symbol(symbol='BTCUSDT', exchange='BINANCE', level_num=5)
# trade_service.add_md_symbol(symbol='DOGEUSDT', exchange='BINANCE')

# setup private config
trade_service.set_ts_connection(ip='localhost', port=8060)  # default setting
trade_service.config_exchange(exchange='BINANCE', trade_type='sandbox')
trade_service.set_apikey(exchange="BINANCE", 
                        key="02SvhZEYG1p92JWdekP75XQKayqfLxmjHWNEfWU1KrCPjJ5xrLcOU1YHZ5SUBVFA", 
                        secret="iKnZvDKMGQuEINhjDX8gbIVJDLl48fV6GFLL5gcFT8Sfj9yxGrnP7uFm7AAVWeFP", 
                        password="",
                        subaccount="")
trade_service.set_fee('BINANCE', 0.0003, 0)
trade_service.add_trade_symbol(symbol='BTCUSDT', exchange='BINANCE', level_num=5)

# setup your strategy
my_strategy = SimpleMakerStrategy()
trade_service.add_strategy(my_strategy)

# start run
try:
    trade_service.run()
except KeyboardInterrupt as err:
    # trade_service.stop()
    print("close strategy and close open orders")
    # trade_service.manager.stop()
    my_strategy.close()
    time.sleep(1)
    print("trade closed")    


import json
import getpass
import time
from functools import wraps


# def create_symbol_info_cfg(symbols):
#         si = dict()
#         for sym in symbols:
#             fp = f'/data/cc/prod/symbol_info/{sym.split(".")[1]}/{sym.split(".")[0]}.json'
#             scfg = json.load(open(fp))
#             si[sym] = scfg['data'][-1][1]
#         return si

# def load_static_symbol_info(port):
#     fp = f'/data/cc/prod/symbol_info/{port.split(".")[1]}/{port.split(".")[0]}.json'
#     scfg = json.load(open(fp))
#     si = scfg['data'][-1][1]
#     return si


def timeit(f):
    @wraps(f)
    def inner(*args, **kw):
        s = time.time_ns()
        f(*args, **kw)
        e = time.time_ns()
        print(f'{f.__name__} finished in {e - s} nanoseconds.')
    return inner





import socket
import json
import time
import datetime
from enum import Enum
from struct import pack, unpack


class NetWorkType(Enum):
    P2P = 1
    BROADCAST = 2
    MULTIBROAD = 3


class MarketService(object):
    def __init__(self, sym_cid_map):
        self.sym_cid_map = sym_cid_map
        self.cid_sym_map = {v: k for k, v in self.sym_cid_map.items()}
        # self.host = socket.gethostname()
        self.ip_port = None
        self.send_to_ip = None
        self.send_to_port = None
        self.symbols = []
        self.exchanges = set()
        return

    def set_control_server(self, ip, port):
        self.control_ip = ip
        self.control_port = port

    def set_network_cfg(self, ip, port, send_type, netcard_name=''):
        self.send_to_ip = ip
        self.send_to_port = port
        self.send_type = send_type
        self.netcard_name = netcard_name

    def add_listen_symbol(self, symbol, exchange, book_level):
        cid = self.cid_sym_map[symbol+'.'+exchange]
        self.symbols.append(
            {'cid': cid, 'port': [symbol, exchange], 'book_level': book_level})
        self.exchanges.add(exchange)

    def send_msg(self, msg, socket):
        message_len = len(msg)
        message_format = '>i' + str(message_len) + 's'
        send_message = pack(message_format, message_len, msg.encode('utf-8'))
        socket.send(send_message)

    def recv_msg(self):
        data = self.server_socket.recv(1024)
        self.server_socket.close()
        if len(data) <= 0:
            return ''

        # print(data)
        length = unpack('i', data[:4])[0]
        resp = json.loads(data[4:length+4].decode('utf-8'))
        # print(resp)
        return resp        

    def run(self):
        self.server_socket = socket.socket()
        self.server_socket.connect((self.control_ip, self.control_port))
        # request new instance and wait for pin and instance port
        msg = {"type": "new_market_instance",
                "cfg": {
                    "instance": {"license_id": "TRAIL001", "license_key": "apifiny123456", "log_path": "/data/cc/log/md_server", "log_level": 1, "name": "trader_service", "book_depth": 5}, 
                    "servers": {"redis_server": "127.0.0.1"}, 
                    "network": {"send_type": self.send_type.value , "network_card_name": self.netcard_name, "local_port": self.send_to_port, "send_to_ip": self.send_to_ip, "send_to_port": self.send_to_port}, 
                    "exchanges": [{"exchange": exch} for exch in self.exchanges], 
                    "symbols": self.symbols}
                    }
        # print(msg)
        msg = json.dumps(msg)
        self.send_msg(msg, self.server_socket)

        resp = self.recv_msg()
        if len(resp) <= 0:
           return
        print(resp)
        resp_type = resp.get("type")

        if resp_type == 'market_instance':
            self._pid = resp.get("data").get("pid")

        while True:
            state = self.check_process()
            # print(state)
            if state == 'ok':
                print("require_market_data_state: ok")
                break
            elif state == 'bind error!':
                print("Failed to start the market data server. Please try another port.")
                break
            else:
                time.sleep(1)
        

    def check_process(self):
        self.server_socket = socket.socket()
        self.server_socket.connect((self.control_ip, self.control_port))
        # request new instance and wait for pin and instance port

        msg = {"type": "require_market_data_state", "pid": self._pid}
        # print(msg)

        msg = json.dumps(msg)
        self.send_msg(msg,self.server_socket)

        resp = self.recv_msg()
        if len(resp) <= 0:
            return "unkonw"
        print(resp)
        resp_type = resp.get("type")
        result = resp.get("result")
        if resp_type == "require_market_data_state":
            return resp.get("data").get("output")
        return "unkonw"

        



import socket
import json
from enum import Enum 
from struct import pack, unpack

class ControlService(object):
    def __init__(self):
        return

    def set_control_server(self, ip, port):
        self.control_ip = ip
        self.control_port = port

    def show_all_trade_servers(self):
        s = socket.socket()
        s.connect((self.control_ip, self.control_port))
        msg = json.dumps({"type": "show_all_trade_instance"})
        self.send_msg(msg, s)
        data = s.recv(1024)
        length = unpack('i', data[:4])[0]
        resp = json.loads(data[4:length+4].decode('utf-8'))
        data = resp.get("data").get("result").split("\n")
        server_list = [{"pid": item.split()[0], "port": item.split()[2]} for item in data if item]
        print("active trade server list: ", server_list)

    def show_all_market_servers(self):
        s = socket.socket()
        s.connect((self.control_ip, self.control_port))
        msg = json.dumps({"type": "show_all_market_instance"})
        self.send_msg(msg, s)
        data = s.recv(1024)
        length = unpack('i', data[:4])[0]
        resp = json.loads(data[4:length+4].decode('utf-8'))
        data = resp.get("data").get("result").split("\n")
        server_list = [{"pid": item.split()[0], "port": item.split()[3]} for item in data if item]
        print("active market server list: ", server_list)

    def kill_server(self, pid):
        s = socket.socket()
        s.connect((self.control_ip, self.control_port))
        msg = json.dumps({"type": "kill_process", "pid":pid})
        self.send_msg(msg, s)
        data = s.recv(1024)
        length = unpack('i', data[:4])[0]
        resp = json.loads(data[4:length+4].decode('utf-8'))
        print(resp)

    def send_msg(self, msg, socket):
        message_len = len(msg)
        message_format = '>i' + str(message_len) + 's'
        send_message = pack(message_format, message_len,msg.encode('utf-8'))
        socket.send(send_message)

        




import time
from struct import pack, unpack

from aats.base.manager import Manager
from aats.base.symbolinfo import SymbolInfo


class TradeService(object):
    def __init__(self, sym_cid_map, verbose=True):
        start = time.time_ns()
        self.sym_cid_map = sym_cid_map
        self.cid_sym_map = {v: k for k, v in self.sym_cid_map.items()}
        self.verbose = verbose
        self.si = dict()
        self.cfg = dict()
        self.cfg['exchanges'] = []
        self.cfg['apikeys'] = dict()
        self.cfg['fees'] = dict()
        self.cfg['symbol_info'] = dict()
        self.cfg['md_symbols'] = []
        self.cfg['trade_symbols'] = []
        self.cfg['verbose'] = self.verbose

    def set_ts_connection(self, ip, port):
        self.cfg['server_ip'] = ip
        self.cfg['server_port'] = port

    def set_md_connection(self, ip, port, send_type):
        self.cfg['send_to_ip'] = ip
        self.cfg['send_to_port'] = port
        self.cfg['md_send_type'] = send_type.value

    def config_exchange(self, exchange, trade_type):
        self.exchange = exchange
        self.trade_type = trade_type
        self.cfg['exchanges'].append(
            {'exchange': self.exchange, 'trader_type': self.trade_type})

    def set_apikey(self, exchange, key, secret, password, subaccount):
        self.exchange = exchange
        self.key = key
        self.secret = secret
        self.password = password
        self.subaccount = subaccount
        self.cfg['apikeys'][exchange] = {
            'key': self.key, 'secret': self.secret, 'pass': self.password, 'subaccount': self.subaccount}

    def set_fee(self, exchange, taker_fee, maker_fee):
        self.taker_fee = taker_fee
        self.maker_fee = maker_fee
        self.cfg['fees'][exchange] = {
            "take": self.taker_fee, "make": self.maker_fee}

    def add_md_symbol(self, symbol, exchange, level_num=5):
        cid = self.cid_sym_map[symbol+'.'+exchange]
        self.cfg['md_symbols'].append({'cid': cid, 'port': [symbol, exchange]})
        self.si[cid] = SymbolInfo(cid, symbol, exchange, level_num)

    def add_trade_symbol(self, symbol, exchange, level_num=5):
        self.trade_symbol = symbol
        self.trade_exchange = exchange
        self.trade_cid = self.cid_sym_map[symbol+'.'+exchange]
        self.cfg['trade_symbols'].append(
            {'cid': self.trade_cid, 'port': [symbol, exchange]})
        self.si[self.trade_cid] = SymbolInfo(
            self.trade_cid, symbol, exchange, level_num)
        # self.si[cid].trade_flag = True

    def add_strategy(self, strategy):
        self.strategy = strategy

    def run(self):
        self.manager = Manager()
        self.manager.subscrib_strategy(self.strategy)
        self.manager.init_manager(self.cfg)
        self.strategy.init_strategy(
            self.si, self.trade_cid, self.trade_symbol, self.trade_exchange, self.manager)
        self.manager.start()
        # self.manager.show_all_trade_instances()

    def stop(self):
        self.manager.stop()




import sys
sys.path.insert(0,'/home/zhangtian/dev/algo-client-api/src/')

from aats.market_service import MarketService
from aats.market_service import NetWorkType

# maintain your unique cid symbol mapping table which needs to be consistent in both market engine and trade engine
sym_cid_map = {
    1001: "BTCUSDT.BINANCE",
    1002: "ETHUSDT.BINANCE",
    1003: "DOGEUSDT.BINANCE",
    1004: "BTCUSDTSWAP.BINANCE_SWAP"
    }

# start market engine
mkt_service = MarketService(sym_cid_map)
mkt_service.set_control_server(ip='127.0.0.1', port=8060)  # default setting
mkt_service.set_network_cfg(ip='127.0.0.1', port=15004, send_type=NetWorkType.P2P, netcard_name = '')
mkt_service.add_listen_symbol('BTCUSDT', 'BINANCE', 5)
mkt_service.add_listen_symbol('ETHUSDT', 'BINANCE', 5)
# mkt_service.add_listen_symbol('DOGEUSDT', 'BINANCE', 5)
# mkt_service.add_listen_symbol('BTCUSDTSWAP', 'BINANCE_SWAP', 5)
mkt_service.run()



from dataclasses import dataclass, field
from itertools import count
from enum import Enum
from aats.base.const import OrderStatus

@dataclass
class Order:
    """
    Order object
    Mandatory field: exchange, symbol, price, size, side, tif (order type, currently support IOC and Post Only)
    """
    exchange: str
    symbol: str
    price: float
    size: float
    side: str
    tif: str
    order_id: int = field(default_factory=count(start=1).__next__, init=False)
    margin: bool = False
    margin_source: str = 'spot'
    fill_size: float = 0 
    fill_price: float = 0
    fill_notional: float = 0
    remote_id: int = 0
    order_status: Enum = OrderStatus.NEW



if __name__ == "__main__":
    # new_order = Order(exchange='BINANCE', symbol='BTCUSDT', price=100, size=0.01, side='buy', tif='pos')
    for i in range(10):
        new_order = Order(exchange='BINANCE', symbol='BTCUSDT', price=100, size=0.01, side='buy', tif='pos')
        print(new_order)